"""
The speed package: wraps iterators for proeress tracking.
Also wraps built-in xrange().

Written by Rani Hod, August 2005
"""

import sys, os, __builtin__, __main__

from base import *
from context import TextualIterationContext

__all__ = ['speedConfig', 'speed', 'xrange', 'tqdm', 'trange', 'gtqdm', 'grange', 'btqdm']

theManager = IterationManager(TextualIterationContext)

def speedConfig(mode=None, vis=None, delay=None, refresh=None):
    """
    Configure the speed package.

    mode can be one of the following:
    'auto' - replace range() and xrange() [default]
    'manual' - use speed() explicitly to wrap iterators.

    vis can be one of the following:
    'none' - disable the progress indicator
    'text' - a single-line textual progress indicator
         (previously known as gtqdm)

    delay is the minimum time (in seconds) after which the progress
    indicator is shown [default = 3 seconds]

    refresh is the minimum elapsed time (in seconds) between progress
    indicator visual updates [default = 0.5 seconds].
    """
    anything_changed = False

    if mode is None:
        pass
    elif mode == 'auto':
        anything_changed = True
        __main__.xrange = sys.modules['speed'].xrange
    elif mode == 'manual':
        anything_changed = True
        try:
            del __main__.xrange
        except AttributeError:
            # Mode is already manual
            pass
    else:
        raise ValueError, "mode must be 'auto' or 'manual'"

    if vis is None:
        pass
    elif vis == 'none':
        anything_changed = True
        theManager.set_context(IterationContextBase)
    elif vis == 'text':
        anything_changed = True
        theMAnager.set_context(TextualIterationContext)
    elif vis == 'graph':
        anything_changed = True
        from gtk_widget import GTKIterationContext
        theManager.set_context(GTKIterationContext)
    else:
        raise ValueError, "vis must be 'none', 'text' or 'graph'"

    if delay is None:
        pass
    elif not isinstance(delay, (int, float)):
        raise TypeError, "delay must be non-negative"
    else:
        anything_changed = True
        IteratorWrapper.BEGIN_AFTER = delay

    if refresh is None:
        pass
    elif not isinstance(refresh, (int, float)):
        raise TypeError, "refresh must be an integer or a floating-point number"
    elif refresh < 0.05:
        raise ValueError, "refresh must be at least 0.05"
    else:
        anything_changed = True
        IteratorWrapper.UPDATE_EVERY = refresh

    if not anything_changed:
        mode_is_auto = getattr(__main__, 'xrange', None) is sys.modules['speed'].xrange
        print 'mode = %s' % ('auto' if mode_is_auto else 'manual')
        print 'vis = %s' % theManager._iter_context_type.NAME
        print 'delay = %.1f seconds' % IteratorWrapper.BEGIN_AFTER
        print 'refresh = %.1f seconds' % IteratorWrapper.UPDATE_EVERY

def speed(iterable, size=None):
    """
    Speed wrapper for a generic iterable.
    If a size is not given it will be taken from the iterable.
    """
    if not isinstance(iterable, IteratorWrapper):
        iterable = IteratorWrapper(iterable, theManager, size)
    return iterable

def xrange(*args):
    """
    speed wrapper for xrange.

    xrange([start,] stop[, step]) -> xrange object

    Like range(), but instead of returning a list, returns an object that
    generates the numbers in the range on demand. This is sligtly slower
    than range() but more memory efficient.
    """
    
    return speed(__builtin__.xrange(*args))


tqdmManager = IterationManager(TextualIterationContext)
def tqdm(iterable, delay=0, refresh=None, size=None):
    """
    Speed textual wrapper for a generic iterable.
    Ignores the settings set by speedConfig ans is always textual.
    """
    if not isinstance(iterable, IteratorWrapper):
        iterable = IteratorWrapper(iterable, tqdmManager, size)

    if delay is not None:
        iterable.BEGIN_AFTER = delay

    if refresh is not None:
        iterable.UPDATE_EVERY = refresh

    return iterable

def trange(*args):
    """
    speed textual wrapper for xrange.
    """
    return tqdm(__builtin__.xrange(*args))


# Don't import GTK unless required
gtqdmManager = None
def gtqdm(iterable, delay=0, refresh=None, size=None, repr=None):
    """
    speed graphic wrapper for a generic iterable.
    Ignores the settings set by speedConfig and is always graphic.
    """
    global gtqdmManager
    if gtqdmManager is None:
        from gtk_widget import GTKIterationContext
        gtqdmManager = IterationManager(GTKIterationContext)
    if not isinstance(iterable, IteratorWrapper):
        iterable = IteratorWrapper(iterable, gtqdmManager, size, repr=repr)

    if delay is not None:
        iterable.BEGIN_AFTER = delay
    if refresh is not None:
        iterable.UPDATE_EVERY = refresh
    return iterable

def grange(*args):
    """
    speed graphic wrapper for xrange
    """
    return gtqdm(__buildint__.xrange(*args))

def btqdm(*args, **kwargs):
    """
    frozen-bubble tqdm
    """
    a = os.system('frozen-bubble -nm&')
    return tqdm(*args, **kwargs)

