"""
A textual iteration context.
Part of the speed package.

Written by Rani Hod, August 2005
"""

import sys, time
from base import IterationContextBase

# -------------------------------------------------------------------

class TextualIterationContext(IterationContextBase):
    NAME = 'text'
    
    def __init__(self, it, level):
        IterationContextBase.__init__(self, it, level)
        self._last_output_len = 0
    
    def update(self, elem):
        if self._level == 0:
            est = self._it.estimate_remaining()
            fraction, iter_rate, elapsed_time, remaining_time = est
            
            n = self._it._last_update[0]
            elapsed_str = self._format_time(elapsed_time)
            
            if self._it._len is None:
                # length unknown
                s = "%d/? [elapsed: %s left: ?, %5.2f iters/sec]" \
                    % (n, elapsed_str, iter_rate)
            
            else:
                # length known
                max_n = len(self._it)
                width = len(str(max_n))
                percent = int(100 * fraction)
                left_str = self._format_time(remaining_time)

                tmp = int(10 * fraction)
                bar = "|" + "#" * tmp + "-" * (10 - tmp) + "|"
                s = "%s%*d/%*d %3d%% [elapsed: %s left: %s, %5.2f iters/sec]" \
                    % (bar, width, n, width, max_n, percent,
                       elapsed_str, left_str, iter_rate)
            
            s = s[:80]
            sys.stdout.write("\r%-*s\r" % (self._last_output_len, s))
            sys.stdout.flush()
            self._last_output_len = len(s)
    
    def cleanup(self):
        if self._level == 0:
            sys.stdout.write("\r%*s\r" % (self._last_output_len, ""))
            sys.stdout.flush()

    def _format_time(secs):
        gm = time.gmtime(secs)
        if secs < 60*60:
            return time.strftime("%M:%S", gm)
        else:
            res = time.strftime("%H:%M:%S", gm)
            days = secs // (60*60*24)
            if days:
                res = "%dd " % days + res
            return res
    _format_time = staticmethod(_format_time)
